From cf56cf0eef8d48ca543aac96d57fdfebe3e8767a Mon Sep 17 00:00:00 2001
From: Takashi Yano <takashi.yano@nifty.ne.jp>
Date: Wed, 27 Jun 2018 19:26:15 +0900
Subject: [PATCH] Fix a bug of perror() which changes the orientation of
 stderr.

* perror.c: Fix the problem that perror() changes the orientation
  of stderr to byte-oriented mode if stderr is not oriented yet.
---
 newlib/libc/stdio/perror.c | 25 +++++++++++++++++++++----
 1 file changed, 21 insertions(+), 4 deletions(-)

diff --git a/newlib/libc/stdio/perror.c b/newlib/libc/stdio/perror.c
index d98e17e19..3328425da 100644
--- a/newlib/libc/stdio/perror.c
+++ b/newlib/libc/stdio/perror.c
@@ -58,6 +58,23 @@ Supporting OS subroutines required: <<close>>, <<fstat>>, <<isatty>>,
 #include <string.h>
 #include "local.h"
 
+static int
+write_err (struct _reent *ptr, const char *s)
+{
+  size_t len = strlen (s);
+  size_t len_out = 0;
+  int fd_err = fileno (_stderr_r (ptr));
+  while (len_out != len)
+    {
+      _ssize_t len1 = _write_r (ptr, fd_err, s+len_out, len-len_out);
+      if (len1 > 0)
+	len_out += len1;
+      else
+	return -1;
+    }
+  return 0;
+}
+
 void
 _perror_r (struct _reent *ptr,
        const char *s)
@@ -68,14 +85,14 @@ _perror_r (struct _reent *ptr,
   _REENT_SMALL_CHECK_INIT (ptr);
   if (s != NULL && *s != '\0')
     {
-      fputs (s, _stderr_r (ptr));
-      fputs (": ", _stderr_r (ptr));
+      write_err (ptr, s);
+      write_err (ptr, ": ");
     }
 
   if ((error = _strerror_r (ptr, ptr->_errno, 1, &dummy)) != NULL)
-    fputs (error, _stderr_r (ptr));
+    write_err (ptr, error);
 
-  fputc ('\n', _stderr_r (ptr));
+  write_err (ptr, "\n");
 }
 
 #ifndef _REENT_ONLY
-- 
2.17.0

